public inbox for glibc-cvs@sourceware.org
help / color / mirror / Atom feed
* [glibc] hurd: readv: Get rid of alloca
@ 2023-06-20 17:15 Samuel Thibault
  0 siblings, 0 replies; only message in thread
From: Samuel Thibault @ 2023-06-20 17:15 UTC (permalink / raw)
  To: glibc-cvs

https://sourceware.org/git/gitweb.cgi?p=glibc.git;h=9e6863a537e66e01f5819dc356c5405a2bc67dc7

commit 9e6863a537e66e01f5819dc356c5405a2bc67dc7
Author: Joe Simmons-Talbott <josimmon@redhat.com>
Date:   Mon Jun 19 10:43:34 2023 -0400

    hurd: readv: Get rid of alloca
    
    Replace alloca with a scratch_buffer to avoid potential stack overflows.
    
    Checked on i686-gnu and x86_64-linux-gnu
    Message-Id: <20230619144334.2902429-1-josimmon@redhat.com>

Diff:
---
 sysdeps/posix/readv.c | 28 ++++++++++++----------------
 1 file changed, 12 insertions(+), 16 deletions(-)

diff --git a/sysdeps/posix/readv.c b/sysdeps/posix/readv.c
index 91208e9894..9cdf571787 100644
--- a/sysdeps/posix/readv.c
+++ b/sysdeps/posix/readv.c
@@ -19,6 +19,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <limits.h>
+#include <scratch_buffer.h>
 #include <stdbool.h>
 #include <sys/param.h>
 #include <sys/uio.h>
@@ -26,9 +27,9 @@
 
 
 static void
-ifree (char **ptrp)
+ifree (struct scratch_buffer *sbuf)
 {
-  free (*ptrp);
+  scratch_buffer_free (sbuf);
 }
 
 /* Read data from file descriptor FD, and put the result in the
@@ -52,20 +53,15 @@ __readv (int fd, const struct iovec *vector, int count)
       bytes += vector[i].iov_len;
     }
 
-  /* Allocate a temporary buffer to hold the data.  We should normally
-     use alloca since it's faster and does not require synchronization
-     with other threads.  But we cannot if the amount of memory
-     required is too large.  */
-  char *buffer;
-  char *malloced_buffer __attribute__ ((__cleanup__ (ifree))) = NULL;
-  if (__libc_use_alloca (bytes))
-    buffer = (char *) __alloca (bytes);
-  else
-    {
-      malloced_buffer = buffer = (char *) malloc (bytes);
-      if (buffer == NULL)
-	return -1;
-    }
+  /* Allocate a temporary buffer to hold the data.  Use a scratch_buffer
+     since it's faster for small buffer sizes but can handle larger
+     allocations as well.  */
+
+  struct scratch_buffer __attribute__ ((__cleanup__ (ifree))) buf;
+  scratch_buffer_init (&buf);
+  if (!scratch_buffer_set_array_size (&buf, 1, bytes))
+    return -1;
+  char *buffer = buf.data;
 
   /* Read the data.  */
   ssize_t bytes_read = __read (fd, buffer, bytes);

^ permalink raw reply	[flat|nested] only message in thread

only message in thread, other threads:[~2023-06-20 17:15 UTC | newest]

Thread overview: (only message) (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2023-06-20 17:15 [glibc] hurd: readv: Get rid of alloca Samuel Thibault

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).